/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ProsthesisObserver
 * @author     JuliaStarke ( julia dot starke at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ProsthesisObserver.h"
#include <ArmarXCore/observers/variant/TimestampVariant.h>

using namespace armarx;


void ProsthesisObserver::onInitObserver()
{
    usingTopic(getProperty<std::string>("ProsthesisTopicName").getValue());
    offeringTopic(getProperty<std::string>("DebugDrawerTopic").getValue());
}


void ProsthesisObserver::onConnectObserver()
{
    debugDrawerPrx = getTopic<DebugDrawerInterfacePrx>(getProperty<std::string>("DebugDrawerTopic").getValue());
}


//void ProsthesisObserver::onDisconnectComponent()
//{

//}


void ProsthesisObserver::onExitObserver()
{

}

PropertyDefinitionsPtr ProsthesisObserver::createPropertyDefinitions()
{
    return PropertyDefinitionsPtr(new ProsthesisObserverPropertyDefinitions(
                                      getConfigIdentifier()));
}

void ProsthesisObserver::reportMotorValues(const ProsthesisMotorValues &motorValues, const Ice::Current &)
{
    ScopedLock lock(dataMutex);
    //TimestampVariantPtr timestampPtr = TimestampVariantPtr::dynamicCast(timestamp);

    std::string name = motorValues.name;
    if (!existsChannel(name))
    {
        offerChannel(name, "Prosthesis motor data");
    }

    offerOrUpdateDataField(name, "name", Variant(name), "Name of the prostesis");
    //offerOrUpdateDataField(name, "device", Variant(device), "Device name");
    offerOrUpdateDataField(name, "position1", Variant(motorValues.position1), "Position motor 2");
    offerOrUpdateDataField(name, "pwm1", Variant(motorValues.pwm1), "PWM motor 2");
    offerOrUpdateDataField(name, "position2", Variant(motorValues.position2), "Position motor 3");
    offerOrUpdateDataField(name, "pwm2", Variant(motorValues.pwm2), "PWM motor 3");
    //offerOrUpdateDataField(name, "timestamp", timestampPtr, "Timestamp");
    //offerOrUpdateDataField(name, "position", Vector2(position1, position2), "Position");
    //offerOrUpdateDataField(name, "pwm", Vector2(pwm1, pwm2), "PWM");

    updateChannel(name);
}
